/*------------------------------------------------------------------------
 Module:        D:\cutmboxD\cutmboxD.c
 Author:        A.P. Veening
 Project:       Cut mbox file into chunks of 500 messages
 State:
 Creation Date: 08-09-2006
 Description:   Cut a Thunderbird mbox file into chunks of about 500 messages,
                each in its own mbox file with consecutive numbers added
				to the base name.

 Linux version by Paolo "Kaosmos"
 Creation Date: 26-07-2007
 Note: Linux version has a parameter to set how many messages the user wants
	in each chunk
------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

FILE *mbox_in;
char line_in[4096];

int get_line_in(void)
{
	int  i = 0;
	int  c;
	int  result = 0;
	char cn;
	char c0;

	memcpy(&cn,"\n",1);
	memcpy(&c0,"\0",1);
	memset(line_in,0,sizeof(line_in));

	c = fgetc(mbox_in);
	if (c == cn) {
		line_in[i] = c;
		i++;
		result = 1;
	}
	while (result == 0) {
		if (c != c0) {
			line_in[i] = c;
			i++;
		}
		if (i < sizeof(line_in)) {
			c = fgetc(mbox_in);
			if (c == EOF) {
				if (i > 0) {
					result = 2;
				} else {
					result = 3;
				}
			} else if (c == cn) {
				line_in[i] = c;
				i++;
				result = 1;
			}

		} else {
			result = 1;
		}
	}
	return result;
}

int main(int argc, char *argv[])
{
	if(argc != 4 || strcmp(argv[1], "-m"))  
	{
		fprintf(stderr, "\nCutMbox for Linux v.0.1\nUsage: cutmbox -m [1-99999] <filename>\nThe parameter -m defines the number of messages in each chunk\n");
		return 1;
	}
	if(strlen(argv[2]) > 5)  
	{
		fprintf(stderr,"\nError: -m parameter too big (max 99999)\n");
		return 1;
	}	
	if (strlen(argv[3]) > 254)
	{
		fprintf(stderr, "\nCutMbox for Linux v.0.1\nUsage: cutmbox -m [1-99999] <filename>\nThe parameter -m defines the number of chunks\n");
		return 2;
  	}

	char file_name_in[255];
	char file_name_base[255];
	char file_name_out[255];
	char file_name_simple[255];
	char seqnma[10];
	long long seqnmn = 0;
	int  rec_status = 5;
	int chunk;
	int msg_cnt = 0;
	char * pch;
	int offset;

	bool file_out_open = false;

	FILE *mbox_out;
		
	chunk =  (int)strtol(argv[2], NULL, 10);
	if (chunk < 1)
	{		
		fprintf(stderr, "\nCutMbox for Linux v.0.1\nUsage: cutmbox -m [1-99999] <filename>\nThe parameter -m defines the number of messages in each chunk\n");
		return 1;
	}
	strcpy(file_name_in, argv[3]);

	/* Strip the path and find the simple filename */	
	pch=strrchr(file_name_in,'/');
	if (pch != NULL) {
		offset = pch-file_name_in+1;
	}
	else {
		offset = 0;
	}
	sprintf (file_name_simple,"%s",&file_name_in[offset]);

	mbox_in = fopen(file_name_in,"r");
	if (mbox_in == NULL) {
		mbox_in = fopen(file_name_in,"r");
		if (mbox_in == NULL) {
			printf("\nFile '%s' was not found or locked.\n",file_name_in);
			exit(1);
		}
	}

	memcpy (file_name_base, file_name_simple, sizeof(file_name_simple));
	strcat (file_name_base, "_");

	rec_status = get_line_in ();
	while (rec_status == 1) {
		if (memcmp(line_in,"From - ",7) == 0) {
			if (msg_cnt == chunk) {
				msg_cnt = 0;
			}
			if (msg_cnt == 0) {

				if (file_out_open) {
					fclose(mbox_out);
				}
				seqnmn++;
    			sprintf(seqnma, "%09lld", seqnmn);
				memcpy (file_name_out, file_name_base, sizeof(file_name_base));
				strcat (file_name_out, seqnma);
				strcat (file_name_out, ".mbx");
				mbox_out = fopen(file_name_out,"w");
				file_out_open = true;
			}
			msg_cnt++;
		}
		if (!file_out_open) {
			// Problems, we want to write, but no file open.
			// Meaning: source file does probably not start with "From - "
			memcpy (file_name_out, file_name_base, sizeof(file_name_base));
			sprintf(seqnma, "%09lld", seqnmn);
			strcat (file_name_out, seqnma);
			strcat (file_name_out, ".mbx");
			mbox_out = fopen(file_name_out,"w");
			file_out_open = true;
		}
		fputs(line_in, mbox_out);
		rec_status = get_line_in ();
	}
	if (rec_status == 2) {
		if (strlen(line_in) > 1) {
			line_in[strlen(line_in) - 1] = 0; // removing artefact x'FF'
			fputs(line_in, mbox_out);
		}
	}

	if (file_out_open) {
		fclose(mbox_out);
	}
	fclose(mbox_in);
	return 0;
}
// Done


